/******************************************
	PURPOSE:	Learn to use timed interrupts on the example of driving a 8x8 LED matrix display
	Created by      Rudy Schlaf for www.makecourse.com
	DATE:		2/2014
*******************************************/


#include <TimerOne.h>//this is a library that uses timer 1 of the arduino to trigger interrupts in certain time intervals

//This defines a matrix defining a smiley face for the 8x8 LED matrix display
#define SMILE { \
    {0, 1, 1, 0, 0, 1, 1, 0}, \
    {0, 1, 1, 0, 0, 1, 1, 0}, \
    {0, 0, 0, 0, 0, 0, 0, 0}, \
    {0, 0, 0, 0, 0, 0, 0, 0}, \
    {1, 0, 0, 0, 0, 0, 0, 1}, \
    {1, 1, 0, 0, 0, 0, 1, 1}, \
    {0, 1, 1, 0, 0, 1, 1, 0}, \
    {0, 0, 1, 1, 1, 1, 0, 0}  \
}

//This defines a matrix defining the letter "A" for the 8x8 LED matrix display
#define A { \
    {0, 0, 0, 1, 1, 0, 0, 0}, \
    {0, 0, 1, 0, 0, 1, 0, 0}, \
    {0, 1, 0, 0, 0, 0, 1, 0}, \
    {0, 1, 0, 0, 0, 0, 1, 0}, \
    {0, 1, 1, 1, 1, 1, 1, 0}, \
    {0, 1, 0, 0, 0, 0, 1, 0}, \
    {0, 1, 0, 0, 0, 0, 1, 0}, \
    {0, 1, 0, 0, 0, 0, 1, 0}  \
}
//NOTE: Driving the 8x8 LED matrix requires 16 digital pins. This requires that we use the analog input pins as digital pins.
//these pins can be defined as digital input/outputs via the pinMode command by using the pin numbers D14 - D19 (= A0 - A5).

//The following is a bit complicated since we have to assign Arduino pins to display pins,
//which then are connected to certain rows and columns; see tutorial video at www.makecourse.com for more info.

/*Arduino pin --> Display pin----->8x8 matrix row and column coordinates (1;1) origin
D2-------------->9---------------------->row 1
D3-------------->10-------------------->col 4
D4-------------->11-------------------->col 6
D5-------------->12--------------------->row 4
D6-------------->13-------------------->col 1
D7-------------->14--------------------->row 2
D8-------------->15-------------------->col 7
D9-------------->16-------------------->col 8
D10------------->4--------------------->col 3
D11------------->3--------------------->col 2
D12------------->2---------------------->row 7
D13------------->1---------------------->row 5
A0 (D14)-------->5---------------------->row 8
A1 (D15)-------->6--------------------->col 5
A2 (D16)-------->7---------------------->row 6
A3 (D17)-------->8---------------------->row 3

*/

//These arrays assign each row and column of the display matrix pin to the appropriate Arduino D-pin as specified in the table above,
//i.e. in row[3], col[5] we would find the Arduino pin numbers that togehter would turn on the (x;y)=(4;6) LED in the matrix

//rows:
const int row[8] = {
  2,7,17,5,13,16,12,14 };//these are the Arduino pins that connect to the anodes (1-8) of the LEDs

//columns:
const int col[8] = {
  6,11,10,3,15,4,8,9  };//these are the Arduino pins that connect to the cathodes (1-8) of the LEDs

volatile byte c,r,flag,counter;//interrupt routine variables, they need to be specified as 'volatile'

// 2-dimensional array that contains the currently 'ON' LEDs in the matrix ('1'='ON'); this is used in the refreshScreen() ISR below:
byte pattern[8][8] = SMILE;//here we put the above defined matrix for the SMILE character into the pattern array.
byte patternSmile[8][8] = SMILE;//here we put the above defined matrix for the SMILE character into the pattern array.
byte patternA[8][8] = A;//here we put the above defined matrix for the A character into the pattern array.

unsigned long previousMillis = 0; //last time we switched patterns [ms]
unsigned long interval = 1000; //time between switching [ms]
int currentPattern = 0; //0 for "SMILE", 1 for "A"

void setup() {
  // initialize the row and column pins as outputs
  // iterate through the pins:
  for (int pin = 0; pin < 8; pin++) {
    // initialize the output pins:
    pinMode(col[pin], OUTPUT);
    digitalWrite(col[pin], HIGH);
    pinMode(row[pin], OUTPUT);
    digitalWrite(row[pin], LOW);  
    // take the col pins (i.e. the cathodes) high and the row pins (anodes) low to ensure that
    // the LEDS are off:   
  }
  Timer1.initialize(100);         // initialize timer1, and set a 100 us second period for the interrupt interval (i.e. the ISR will be called
  //every 100 us - this seems to be a good frequency to achieve a flicker-free LED display.
  //experiment with this parameter. If it gets too small the ISR starts 'eating up' all the processor time, and the main loop becomes very slow
  Timer1.attachInterrupt(refreshScreen);  // attaches the refreshScreen() function as 'Interrupt Service Routine' (ISR) to the interrupt
  //this means that every time 100 us have passed, the refreshScreen() routine will be called.
}

//main loop...here we can simply busy ourselves with changing the pattern[][] array; nothing deals with the LED display.
//this is all handled via the ISR
void loop() {
  unsigned long currentMillis = millis(); //current time microcontroller has been on
  if(currentMillis - previousMillis > interval) //is the current time minus the last time the pattern switched greater than the interval time?
  {
    if(currentPattern == 0) //if the current pattern is the smile, switch it to the A
    {
      for(int i = 0; i < 8; i++)
      {
        for(int j = 0; j < 8; j++)
        {
          pattern[i][j] = patternA[i][j];
        }
      }
      currentPattern = 1;
    }
    else //otherwise if it is A switch it to the smile
    {
      for(int i = 0; i < 8; i++)
      {
        for(int j = 0; j < 8; j++)
        {
          pattern[i][j] = patternSmile[i][j];
        }
      }
      currentPattern = 0;
    }
    previousMillis = currentMillis;
  }
}

